!+ Calculate optical depths
!
SUBROUTINE OPDEP &
(knpf, kchan, kprof, knchpf, ksat, opdp, opdpref, zopdpref, &
xxm, xxw, xxo, xxc, debyeprof)
!
!    This software was developed within the context of 
!    the EUMETSAT Satellite Application Facility on 
!    Numerical Weather Prediction (NWP SAF), under the 
!    Cooperation Agreement dated 25 November 1998, between 
!    EUMETSAT and the Met Office, UK, by one or more partners 
!    within the NWP SAF. The partners in the NWP SAF are 
!    the Met Office, ECMWF, KNMI and MeteoFrance.
! 
!    Copyright 2002, EUMETSAT, All Rights Reserved.
!
! Description:
!  To calculate optical depths for a number of channels
!  and profiles from every pressure level to space.
!
! Method:
!          for gases: ECMWF TECH MEM 345.
!          for cloud liquid water: see comments in the code
!   
!
! Current Code Owner: SAF NWP
!
! History:
! Version   Date        Comment
! -------   ----        -------
!          04/03/92.    Change to +ve opt dep and add traps
!          04/09/92.    For version 2. move io to arguments, add sat index,
!                         change inner loops (faster).
!            7/7/97     Added ozone, extended water vapour 
!           15/8/97     Correct bug in adjoint of OPDEP (Erik)
!            6/7/98     Improved vectorisation of routines (Roger)
!            7/9/99     Added LWP + SSM/I logic (English/Saunders)
!        01/05/2000     F90 code 
!        23/03/2001     New coef. format, new channel numbers (P. Brunel)
!        09/08/2001     New predictors (Marco Matricardi)
!        14/09/2001     Changed to use structures.  Andrew Collard.
!        21/08/2002     Added test on layer optical depth.  (P. Brunel)
!        21/08/2002     Make test on layer optical depth consistent with
!                       K, AD and TL.               Andrew Collard.
!
! Code Description:
!   Language:          Fortran 90.
!   Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code".
!
! Declarations:
! Modules used:
!
  USE MOD_CPARAM, ONLY : &
  ! Imported Scalar Variables with intent (in):
  jplev   ,&    ! No. of pressure levels
  !jphir   ,&    ! Max. no. of hirs channels
  !jpmsu   ,&    ! Max. no. of msu channels
  njplev  ,&    ! No. of pressure levels
  njpcofm ,&    ! Mixed gas coeffs (max
  njpcofw ,&    ! Water vapour coeffs (max
  njpcofo ,&    ! Ozone coeffs
  nmwcldtop     ! Upper level for lwp calcs

  Use MOD_FMV, Only : &
  ! imported arrays with intent (out):
  sensor ,&     ! sensor number for each triplet
  fmv_gas       ! number of active gases

  USE MOD_TAUCFN, ONLY : &
       ! imported structure with intent (out):
       TransCoeffs
         ! Structure contains:   
         !    cfm  :  mixed gas coeffs
         !    cfw  :  water vapour coeffs 
         !    cfo  :  ozone coeffs

  Use MOD_TOVCHN, Only : &
       ! Imported structure with intent (in):
       ChanConstants
         ! Structure contains:   
         !    freq   :   channel frequencies in ghz
         !    gamma  :   "gamma factor" transmittance corrections

  IMPLICIT NONE
  
  ! Subroutine arguments
  ! Scalar arguments with intent(in):
  INTEGER , INTENT(in) :: knpf   ! Number of profiles
  INTEGER , INTENT(in) :: knchpf ! Number of radiances (chans * profs).
  INTEGER , INTENT(in) :: ksat   ! Satellite index.

  ! Array  arguments with intent(in):
  INTEGER , INTENT(in) :: kchan(knchpf) ! Channel indices for each rad
  INTEGER , INTENT(in) :: kprof(knchpf) ! Profile indices for each rad
  REAL, INTENT(in) :: xxm(njplev,knpf,njpcofm) ! Func. of profile for mixed gas
  REAL, INTENT(in) :: xxw(njplev,knpf,njpcofw) ! ...... for water vapour
  REAL, INTENT(in) :: xxo(njplev,knpf,njpcofo) ! ...... for ozone 
  REAL, INTENT(in) :: xxc(njplev,knpf,1)       ! ...... for cloud liquid water 
  REAL, INTENT(in) :: debyeprof(5,njplev,knpf) ! Functions of debye terms of 
                                               !   profile for cloud calc

  ! Array  arguments with intent(out):
  REAL, INTENT(out) :: opdp(knchpf,njplev)    ! Optical depth from press 
                                              !    level to space
  REAL, INTENT(out) :: opdpref(knchpf,njplev) ! same as OPDP but without 
                                              !   control check
  REAL, INTENT(out) :: zopdpref(njplev,knchpf) ! ZOPDP before sign check  
     
  ! Local scalars:
  INTEGER :: j		! channel indice
  INTEGER :: jl		! level indice
  INTEGER :: ich	! channel number
  INTEGER :: ipf	! profile number

  ! Cloud Liquid Water local variables
  REAL :: zf, zfsq, z34dif, z45dif, z1sq, z2sq, z1div, z2div
  REAL :: z1den, z2den, zastar, z1prod, z2prod, z3prod, z4prod
  REAL :: zbstar, zbstarsq, za2star, za2starsq, zdiv, zgstar
  REAL :: z1fsqz1sq, z2fsqz2sq
  
  ! Local arrays:
  REAL :: opdpa(knchpf)        ! Workspace used in calculating opdp
  REAL :: zopdp(njplev,knchpf) ! Workspace used in calculating opdp
  REAL :: zero
  !- End of header ------------------------------------------------------
  !
  !
  opdp(:,:) = 0.0
  opdpa(:)  = 0.0
  zero = 0.0

  !     -----------------------------------------------------------------
  !*         1.   Calculate layer gaseous optical depths (* -1).
  !               --------- ----- ------- ------- ------ -------
  DO j = 1, knchpf
     ich     = kchan(j)
     ipf     = kprof(j)
     !
     DO jl = 1, jplev
        !
        !*         1.1  Uniformly mixed gases
        !
        ! All channels use predictors from Matricardi 2000 
        zopdp(jl,j)=                             &
             TransCoeffs(ksat) % cfm(jl,ich,1)*xxm(jl,ipf,1)+   &
             TransCoeffs(ksat) % cfm(jl,ich,2)*xxm(jl,ipf,2)+   &
             TransCoeffs(ksat) % cfm(jl,ich,3)*xxm(jl,ipf,3)+   &
             TransCoeffs(ksat) % cfm(jl,ich,4)*xxm(jl,ipf,4)+   &
             TransCoeffs(ksat) % cfm(jl,ich,5)*xxm(jl,ipf,5)+   &
             TransCoeffs(ksat) % cfm(jl,ich,6)*xxm(jl,ipf,6)+   &
             TransCoeffs(ksat) % cfm(jl,ich,7)*xxm(jl,ipf,7)+   &
             TransCoeffs(ksat) % cfm(jl,ich,8)*xxm(jl,ipf,8)+   &
             TransCoeffs(ksat) % cfm(jl,ich,9)*xxm(jl,ipf,9)+   &
             TransCoeffs(ksat) % cfm(jl,ich,10)*xxm(jl,ipf,10)
     ENDDO
     !
     !*         1.2  Water vapour
     !
     ! All channels use predictors from Matricardi 2000 
     DO jl = 1, jplev
           zopdp(jl,j)=zopdp(jl,j)+                 &
              TransCoeffs(ksat) % cfw(jl,ich,1)*xxw(jl,ipf,1)+   &
              TransCoeffs(ksat) % cfw(jl,ich,2)*xxw(jl,ipf,2)+   &
              TransCoeffs(ksat) % cfw(jl,ich,3)*xxw(jl,ipf,3)+   &
              TransCoeffs(ksat) % cfw(jl,ich,4)*xxw(jl,ipf,4)+   &
              TransCoeffs(ksat) % cfw(jl,ich,5)*xxw(jl,ipf,5)+   &
              TransCoeffs(ksat) % cfw(jl,ich,6)*xxw(jl,ipf,6)+   &
              TransCoeffs(ksat) % cfw(jl,ich,7)*xxw(jl,ipf,7)+   &
              TransCoeffs(ksat) % cfw(jl,ich,8)*xxw(jl,ipf,8)+   &
              TransCoeffs(ksat) % cfw(jl,ich,9)*xxw(jl,ipf,9)+   &
              TransCoeffs(ksat) % cfw(jl,ich,10)*xxw(jl,ipf,10)+ & 
              TransCoeffs(ksat) % cfw(jl,ich,11)*xxw(jl,ipf,11)+   &  
              TransCoeffs(ksat) % cfw(jl,ich,12)*xxw(jl,ipf,12)+   & 
              TransCoeffs(ksat) % cfw(jl,ich,13)*xxw(jl,ipf,13)+   & 
              TransCoeffs(ksat) % cfw(jl,ich,14)*xxw(jl,ipf,14)+   & 
              TransCoeffs(ksat) % cfw(jl,ich,15)*xxw(jl,ipf,15) 
     ENDDO
     !
     ! New: ozone ==> if fmv_gas(ksat) > 2 
     ! can be replaced in the future by a flag on each channel
     ! indicating the absorption gases.
     IF( fmv_gas(ksat) > 2 ) THEN
        ! All ozone channels:  use predictors from Matricardi 2000 
        DO jl = 1, jplev
           !*         1.3  Ozone  (only for IR channels)
           !
           zopdp(jl,j)=zopdp(jl,j)+                 &
                TransCoeffs(ksat) % cfo(jl,ich,1)*xxo(jl,ipf,1)+   &
                TransCoeffs(ksat) % cfo(jl,ich,2)*xxo(jl,ipf,2)+   &
                TransCoeffs(ksat) % cfo(jl,ich,3)*xxo(jl,ipf,3)+   &
                TransCoeffs(ksat) % cfo(jl,ich,4)*xxo(jl,ipf,4)+   &
                TransCoeffs(ksat) % cfo(jl,ich,5)*xxo(jl,ipf,5)+   &
                TransCoeffs(ksat) % cfo(jl,ich,6)*xxo(jl,ipf,6)+   &
                TransCoeffs(ksat) % cfo(jl,ich,7)*xxo(jl,ipf,7)+   &
                TransCoeffs(ksat) % cfo(jl,ich,8)*xxo(jl,ipf,8)+   &
                TransCoeffs(ksat) % cfo(jl,ich,9)*xxo(jl,ipf,9)+   &
                TransCoeffs(ksat) % cfo(jl,ich,10)*xxo(jl,ipf,10)  +  &
                TransCoeffs(ksat) % cfo(jl,ich,11)*xxo(jl,ipf,11) 
        ENDDO
     ENDIF

     DO jl = 1, jplev
        !   
        !*         1.4   Cloud liquid water  (only for MW channels)
        !   
        IF (sensor(ksat) /=2 .OR. jl < nmwcldtop .OR. xxc(1,ipf,1) < 0.0) THEN
              zopdp(jl,j) = zopdp(jl,j)
        ELSE
 ! Calculate imaginary part of complex refractive index without using 
 ! complex Fortran as this is computationally expensive to run.
 !
 ! The refractive index is based on Liebe, Hufford and Manabe (1991) Int. J.
 ! Infrared and Millimeter Waves, 12, 659-671 except that for cloud
 ! with T>273.15 K the coefficients for this model have been recalculated to
 ! fit the measurements made at PIOM laboratory (Lamkaouchi, Balana and Ellison
 ! (1997) New permittivity data for sea water (30-100 GHz), Extension of ESA
 ! report 11197/94/NL/CN. The new model is described in English, Poulsen and
 ! Smith (1999), Proceedings of ATOVS workshop at ECMWF 2-5/9/99.
 !
 ! The code is written in a step by step format to ease interpretation of
 ! tangent-linear and adjoint code.

              zf       = ChanConstants(ksat) % freq(ich)
              zfsq     = zf*zf
              z34dif   = debyeprof(3,jl,ipf) - debyeprof(4,jl,ipf)
              z45dif   = debyeprof(4,jl,ipf) - debyeprof(5,jl,ipf)
              z1sq     = debyeprof(1,jl,ipf)*debyeprof(1,jl,ipf)
              z2sq     = debyeprof(2,jl,ipf)*debyeprof(2,jl,ipf)
              z1fsqz1sq= zfsq + z1sq
              z2fsqz2sq= zfsq + z2sq
              z1div    = 1.0/z1fsqz1sq
              z2div    = 1.0/z2fsqz2sq
              z1den    = z34dif*z1div
              z2den    = z45dif*z2div
              zastar   = debyeprof(3,jl,ipf) - zfsq*(z1den+z2den)
              z1prod   = z34dif*debyeprof(1,jl,ipf)
              z2prod   = z1prod*z1div
              z3prod   = z45dif*debyeprof(2,jl,ipf)
              z4prod   = z3prod*z2div
              zbstar   = -zf*(z2prod + z4prod)
              zbstarsq = zbstar*zbstar
              za2star  = zastar + 2.0
              za2starsq= za2star*za2star
              zdiv     = za2starsq + zbstarsq
              zgstar   = -3.0*zbstar/zdiv
              zopdp(jl,j) = zopdp(jl,j) - 1.5*zf*zgstar*xxc(jl,ipf,1) 
        ENDIF
     ENDDO! End of level loop
  ENDDO! End of channel/profile loop

  ! save old zopdp-value in zopdpref for use in ad,k,tl
  zopdpref(:,:) = zopdp(:,:)

  where( zopdp(:,:) > 0.0 )
     zopdp = 0.0
  end where

  DO jl = 1, jplev
     DO j = 1, knchpf
        ich = kchan(j)
        !     -----------------------------------------------------------------
        !          2.   Compute layer to space optical depths
        !               ------- ----- -- ----- ------- ------
        opdpa(j)   = opdpa(j)   + zopdp(jl,j)
        opdp(j,jl) = opdp(j,jl) + opdpa(j)
        !     -----------------------------------------------------------------
        !*         3.   Apply gamma corrections and change to +ve opt depth.
        !               ----- ----- ----------- --- ------ -- --- --- ------
        opdp(j,jl) = -opdp(j,jl)*ChanConstants(ksat) % gamma(ich)
        !               save old opdp-value in opdpref for use in ad,k,tl
        opdpref(j,jl) = opdp(j,jl)
        !     ------------------------------------------------------------------
        !          4.  Check for reasonable value
        !              ----- --- ---------- -----
        IF(jl == 1) THEN
           opdp(j,jl) = MAX(zero,opdp(j,jl))
        ELSE
           opdp(j,jl) = MAX(opdp(j,jl-1),opdp(j,jl))
        ENDIF
        !
     ENDDO! End of channel/profile loop
  ENDDO! End of level loop
  ! 
  RETURN
END SUBROUTINE OPDEP
