!+ Tangent linear of fast radiative transfer model.
!
SUBROUTINE RTTOVTL &
     (knpf, klenpf,                                              &
     ppres, pangl, pangs, ksurf, ksat, knchpf,                   &
     kchan, kprof, pav, psav, pssv, pcv, pemis,                  &
     pav_d, psav_d, pssv_d, pcv_d, pemis_d, radov_d,             &
     prad, ptb, radov, lcloud, ifail)
!
!    This software was developed within the context of 
!    the EUMETSAT Satellite Application Facility on 
!    Numerical Weather Prediction (NWP SAF), under the 
!    Cooperation Agreement dated 25 November 1998, between 
!    EUMETSAT and the Met Office, UK, by one or more partners 
!    within the NWP SAF. The partners in the NWP SAF are 
!    the Met Office, ECMWF, KNMI and MeteoFrance.
! 
!    Copyright 2002, EUMETSAT, All Rights Reserved.
!
! Description:
! to compute multi-channel radiances and brightness
! temperatures for many profiles.
!
!  The terms "constant" and "variable" are employed here in the sense used in 
!  variational analysis, i.e. an input variable is a parameter with respect to 
!  which a gradient will be calculated in the associated
!  tangent linear (TL) and adjoint (AD) routines. 
!
!  In all TL code all variables used for the direct model 
!  (input or output) are suffixed with "_d"
!
! Method
! see Matricardi et al QJ 2002 
! see Saunders et al QJ 1999 
! see: ECMWF Technical Memoranda 176/282/345 (Available from ECMWF)
!
! Current Code Owner: SAF NWP
!
! History:
! Version   Date        Comment
! -------   ----        -------
!          13/8/92.  For version 2.
!                    ksat added to argument list; ssu included;
!                    internal changes to move big arrays from commons to
!                    arguments and to introduce taskcommons
!           8/7/97   added ozone and extended water vapour in control vector
!        01/05/2000     F90 code 
!        21/08/2000  Interface to rtint changed to include pref (surface reflectivity).
!                    (Stephen English)
!        31/01/2001  More cloud computations. stored in radov (F. Chevallier)
!        12/2/2001   knav etc arrays removed from call (R. Saunders)
!        18/01/2002  Thread safe (D.Salmond)
!        11/03/2004  Fix OPDEP AD,TL,K (A Collard)
!
! Code Description:
!   Language:          Fortran 90.
!   Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code".
!
! Declarations:
! Modules used:
!
  USE MOD_CPARAM, ONLY : &
  ! Imported Scalar Variables with intent (in):
  jppf     ,&   ! Max no. profiles
  jplev  ,&     ! No. of pressure levels
  jpchpf ,&     ! Max no. of profs * chans used
  njpnsat  ,&   ! Total max sats to be used
  njplev   ,&   ! No. of pressure levels
  njpnav   ,&   ! No. of profile variables
  njpnsav  ,&   ! No. of surface air variables
  njpnssv  ,&   ! No. of skin variables
  njpncv   ,&   ! No. of cloud variables
  njpcofm  ,&   ! Mixed gas coeffs (max)
  njpcofw  ,&   ! Water vapour coeffs (max)
  njpcofo       ! Ozone coeffs (max)

  !
  ! DEPENDS ON: PRFIN.o
  ! DEPENDS ON: OPDEP.o
  ! DEPENDS ON: RTTAU.o
  ! DEPENDS ON: RTINT.o
  ! DEPENDS ON: PRFINTL.o
  ! DEPENDS ON: OPDEPTL.o
  ! DEPENDS ON: RTTAUTL.o
  ! DEPENDS ON: RTINTTL.o
  IMPLICIT NONE
  ! Subroutine arguments
  ! Scalar arguments with intent(in):
  INTEGER , INTENT(in) :: knpf           ! Number of profiles
  INTEGER , INTENT(in) :: klenpf         ! Length of input  profile vectors 
  INTEGER , INTENT(in) :: ksat           ! Satellite index (see rttvi)
  INTEGER , INTENT(in) :: knchpf         ! Number of output radiances
                                         !    (= channels used * profiles) 
  LOGICAL, INTENT(in) :: lcloud          ! switch for cloud computations
  ! Array  arguments with intent(in):
  INTEGER , INTENT(in) :: kchan(knchpf)  ! Channel indices
  INTEGER , INTENT(in) :: kprof(knchpf)  ! Profiles indices
  INTEGER , INTENT(in) :: ksurf(knpf)    ! Surface type index
  REAL , INTENT(in)    :: ppres(njplev)  ! Pressure levels (hpa) of 
                                         !   atmospheric profile vectors
  REAL , INTENT(in)    :: pangl(knpf)    ! Satellite local zenith angle (deg)
  REAL , INTENT(in)    :: pangs(knpf)    ! Solar zenith angle at surface (deg)
  ! Input Variables
  REAL , INTENT(in)    :: pav_d(njplev,njpnav,knpf)! Atmosp. profile variables
  REAL , INTENT(in)    :: psav_d(njpnsav,knpf)     ! Surface air variables
  REAL , INTENT(in)    :: pssv_d(njpnssv,knpf)     ! Surface skin variables
  REAL , INTENT(in)    :: pcv_d(njpncv,knpf)       ! Cloud variables
  ! Input TL variables
  REAL , INTENT(in)    :: pav(njplev,njpnav,knpf)  ! Atmosp. profile variables
  REAL , INTENT(in)    :: psav(njpnsav,knpf)       ! Surface air variables
  REAL , INTENT(in)    :: pssv(njpnssv,knpf)       ! Surface skin variables
  REAL , INTENT(in)    :: pcv(njpncv,knpf)         ! Cloud variables
  !
  ! Array  arguments with intent(inout):
  REAL , INTENT(inout) :: pemis(knchpf)            ! Surface emissivities
  REAL , INTENT(inout) :: pemis_d(knchpf)          ! Surface emissivities
  !
  ! Scalar arguments with intent(out):
  ! Array  arguments with intent(out):
  INTEGER , INTENT(out) :: ifail(knpf,njpnsat)  !  return flag 
              !     0 = input profile OK
              ! 11-19 = outside profile limits
              !    11 =  temp profile
              !    12 =  specific humidity profile
              !    13 =  ozone profile
              !    14 =  surface temp profile
              !    15 =  surface specific humidity profile
              !    16 =  surface wind 
              ! 20-29 = unphysical profile 
              !    20 =  input pressure levels wrong
              !    21 =  temp profile
              !    22 =  specific humidity profile
              !    23 =  ozone profile
              !    24 =  surface temp profile
              !    25 =  surface specific humidity profile
              !    26 =  surface wind 
              !    27 =  surface pressure

  ! TL variables output
  REAL , INTENT(out)    :: prad(knchpf)         ! radiances (mw/cm-1/ster/sq.m)
  REAL , INTENT(out)    :: ptb(knchpf)          ! brightness temperatures (K)
  ! RT quantities for possible cloud computations outside RTTOV
  !   in mw/m2/sr/cm-1 
  REAL , INTENT(out)    :: radov_d(knchpf,2*njplev+2) 
  REAL , INTENT(out)    :: radov(knchpf,2*njplev+2)

  ! Local arrays:
 REAL  :: xpath(jppf)  ! Secant of viewing path angle at surface
 REAL  :: xpath1(jppf) ! XPATH - 1
 REAL  :: sqtpth(jppf) ! Sqrt of XPATH
 REAL  :: xpaths(jppf) ! Secant of solar zenith angle at surface
 REAL  :: snad2(jppf)  ! Sine nadir angle squared
 REAL  :: cnad2(jppf)  ! Cos nadir angle squared
 REAL  :: czen(jppf)   ! Cosine zenith angle
 REAL  :: czen2(jppf)  ! Cos zenith angle squared
 REAL  :: szen2(jppf)  ! Sine zenith angle squared
 INTEGER :: nstype(jppf)   ! Surface type index; 1=sea, 2=land.
 INTEGER :: nlevsf(jppf)   ! Index of nearest std press level at/below surface
 REAL     :: fracps(jppf)   ! Fraction of std press level interval by which
                                ! surface is above level NLEVSF.
 REAL     :: fracps_d(jppf) ! Fraction of std press level interval by which
                                ! surface is above level NLEVSF.
 REAL     :: surfw(jppf)    ! Surface windspeed m/s
 REAL     :: surfw_d(jppf)    ! Surface windspeed m/s
 INTEGER :: nlevcd(jppf)   ! Index of nearest std press level at/below
                            ! cloud top.
 REAL     :: fracpc(jppf)   ! Fraction of std press level interval by which
                            ! cloud is above level NLEVCD.
 REAL     :: fracpc_d(jppf)   ! Fraction of std press level interval by which
                           ! cloud is above level NLEVCD.
 REAL  :: ts(jppf)          ! Surface skin temperature in K.
 REAL  :: ts_d(jppf)          ! Surface skin temperature in K.
 REAL  :: temp(jplev,jppf)  ! Temperature profile in K.
 REAL  :: temp_d(jplev,jppf)  ! Temperature profile in K.
 REAL  :: wmix(jplev,jppf)  ! Specific humidity profile in g/kg
 REAL  :: wmix_d(jplev,jppf)  ! Specific humidity profile in g/kg
 REAL  :: ozon(jplev,jppf)  ! Ozone profile in ppmv
 REAL  :: ozon_d(jplev,jppf)  ! Ozone profile in ppmv
 REAL  :: cldw(jplev,jppf)  ! Cloud liquid water in kg/kg
 REAL  :: cldw_d(jplev,jppf)  ! Cloud liquid water in kg/kg
 REAL  :: ta(jppf)          ! Surface air temperature in K.
 REAL  :: ta_d(jppf)          ! Surface air temperature in K.
 REAL  :: wmixs(jppf)       ! Surface specific humidity in g/kg
 REAL  :: wmixs_d(jppf)       ! Surface specific humidity in g/kg
 REAL  :: surfp(jppf)       ! Surface pressure in mb=hPa.
 REAL  :: surfp_d(jppf)       ! Surface pressure in mb=hPa.
 REAL  :: cldp(jppf)        ! Cloud-top pressure in mb=hPa.
 REAL  :: cldp_d(jppf)        ! Cloud-top pressure in mb=hPa.
 REAL  :: cldf(jppf)        ! Fractional (ir) cloud cover.
 REAL  :: cldf_d(jppf)        ! Fractional (ir) cloud cover.
 REAL  :: emis(jpchpf)      ! Surface emissivity
 REAL  :: emis_d(jpchpf)      ! Surface emissivity
 REAL  :: emis1_d(jpchpf)      ! Surface emissivity

  ! TL arrays
  REAL :: opdp(knchpf,njplev)        ! Optical depth from press level to space 
  REAL :: xxm(njplev,knpf,njpcofm)   ! Functions of profile for mixed gas 
  REAL :: xxw(njplev,knpf,njpcofw)   ! Functions of profile for water vapour 
  REAL :: xxo(njplev,knpf,njpcofo)   ! Functions of profile for ozone 
  REAL :: xxc(njplev,knpf,1)         ! Funct. of profile for cloud liquid water

  REAL :: b(knchpf,njplev)           ! PLANCK functions for temp. profiles
  REAL :: ba(knchpf)                 ! PLANCK functions for surface air temp.
  REAL :: bs(knchpf)                 ! PLANCK functions for surface skin temp.
  REAL :: bdt(knchpf,njplev)         ! Stores upwelling radiation from
                                     !   atmosphere above each level.
  REAL :: bdtr(knchpf,njplev)        ! Stores downwelling radiation at
                                     !   each level from atmosphere above
  REAL :: fcld(knchpf)               ! Effective fractional cloud 
                                     !   cover for each chan * prof
  REAL :: radcl(knchpf)              ! Clear column radiances
  REAL :: tbcl(knchpf)               ! Clear column brightness temp. (K)
      
  REAL :: debyeprof(5,njplev,knpf)   ! Functions of debye terms of 
                                     !   profile for cloud calc
  REAL :: debyesv(5,njplev,knpf)     ! Functions of debye terms of 
                                     !   surface t for emiss calc
  REAL :: rado(knchpf)              ! overcast radiance at given
                                     !   cloud top in mw/m2/sr/cm-1
  REAL :: tau(knchpf,njplev)        ! transmittance from each 
                                     !   standard pressure level
  REAL :: tausfc(knchpf)            ! transmittance from surface

  REAL :: prefl(knchpf)              ! Surface reflectivities

  ! Direct model arrays
  REAL :: opdp_d(knchpf,njplev)      ! Optical depth from press level to space 
  REAL :: opdpref_d(knchpf,njplev)   ! same as OPDP but without control check
  REAL :: zopdpref_d(njplev,knchpf)  ! same as OPDP but without control check
  REAL :: xxm_d(njplev,knpf,njpcofm) ! Functions of profile for mixed gas 
  REAL :: xxw_d(njplev,knpf,njpcofw) ! Functions of profile for water vapour 
  REAL :: xxo_d(njplev,knpf,njpcofo) ! Functions of profile for ozone 
  REAL :: xxc_d(njplev,knpf,1)       ! Funct. of profile for cloud liquid water

  REAL :: b_d(knchpf,njplev)         ! PLANCK functions for temp. profiles
  REAL :: ba_d(knchpf)               ! PLANCK functions for surface air temp.
  REAL :: bs_d(knchpf)               ! PLANCK functions for surface skin temp.
  REAL :: bdt_d(knchpf,njplev)       ! Stores upwelling radiation from
                                     !   atmosphere above each level.
  REAL :: bdtr_d(knchpf,njplev)      ! Stores downwelling radiation at
                                     !   each level from atmosphere above
  REAL :: bdtrc_d(knchpf,njplev)     ! Stores downwelling radiation 
                                     !   below overcast cloud
  REAL :: fcld_d(knchpf)             ! Effective fractional cloud 
                                     !   cover for each chan * prof
  REAL :: radcl_d(knchpf)            ! Clear column radiances
  REAL :: tbcl_d(knchpf)             ! Clear column brightness temp. (K)
      
  REAL :: debyeprof_d(5,njplev,knpf) ! Functions of debye terms of 
                                     !   profile for cloud calc
  REAL :: debyesv_d(5,njplev,knpf)   ! Functions of debye terms of 
                                     !   surface t for emiss calc
  REAL :: rado_d(knchpf)            ! overcast radiance at given
                                     !   cloud top in mw/m2/sr/cm-1
  REAL :: tau_d(knchpf,njplev)      ! transmittance from each 
                                     !   standard pressure level
  REAL :: tausfc_d(knchpf)          ! transmittance from surface
  REAL :: prad_d(knchpf)             ! radiances (mw/cm-1/ster/sq.m)
  REAL :: ptb_d(knchpf)              ! brightness temperatures (K)
  REAL :: prefl_d(knchpf)            ! Surface reflectivities

!- End of header ------------------------------------------------------

  !-----------------------------------------------------------------
  ! Repeat direct calculation
  !-----------------------------------------------------------------
  !*         1.   Set up profile variables.
  !               --- -- ------- ---------
  CALL prfin(ppres, pav_d, psav_d, pssv_d, pcv_d, pemis_d, knpf, klenpf, &
       &  knchpf, pangl, pangs, ksurf, ksat,                             &
       & xxm_d, xxw_d, xxo_d, xxc_d, debyeprof_d, debyesv_d,&
       & nstype,&
       & xpath,xpath1,sqtpth,xpaths,snad2,cnad2,czen,czen2,szen2,&
       & nlevsf,fracps_d,surfw_d,nlevcd,fracpc_d,&
       & ts_d,temp_d,wmix_d,ozon_d,cldw_d, &
       & emis_d,ta_d,wmixs_d,surfp_d,cldp_d,cldf_d,&
       & ifail)

  !
  !-----------------------------------------------------------------
  !*         2.   Calculate optical depths.
  !               --------- ------- ------
  CALL opdep(knpf, kchan, kprof, knchpf, ksat,&
       & opdp_d, opdpref_d, zopdpref_d, &
       & xxm_d, xxw_d, xxo_d, xxc_d, debyeprof_d)
  !
  !-----------------------------------------------------------------
  !*         3.   Calculate transmittances on *rt* model levels.
  !               --------- -------------- -- ---- ----- ------
  CALL rttau(kprof, knchpf, opdp_d, tau_d, tausfc_d,nlevsf,fracps_d)
  !
  !-----------------------------------------------------------------
  !*         4.   Integrate *rt* equation.
  !               --------- ---- --------
  CALL rtint(kchan, kprof, knchpf, lcloud, prad_d, ptb_d,&
       & ksat, pemis_d, prefl_d, tau_d, tausfc_d, b_d, ba_d, bs_d,&
       & radov_d, rado_d, bdt_d, bdtrc_d, bdtr_d, fcld_d, radcl_d, tbcl_d,&
       & nstype,&
       & nlevsf,fracps_d,xpath,snad2,cnad2,czen,czen2,szen2,surfw_d,&
       & nlevcd,fracpc_d,&
       & temp_d,ta_d,ts_d,cldf_d,emis_d,emis1_d)

  !
  !-----------------------------------------------------------------
  !
  ! Initialize TL variables
  !
  xxm(:,1:knpf,:)=0.
  xxw(:,1:knpf,:)=0.
  xxo(:,1:knpf,:)=0.
  xxc(:,1:knpf,:)=0.
  cldw(:,1:knpf)=0.
  wmix(:,1:knpf)=0.
  ozon(:,1:knpf)=0.
  temp(:,1:knpf)=0.
  fracps(1:knpf)=0.
  fracpc(1:knpf)=0.
  cldf  (1:knpf)=0.
  ta    (1:knpf)=0.
  ts    (1:knpf)=0.
  b   (1:knchpf,:)=0.
  bdt (1:knchpf,:)=0.
  bdtr(1:knchpf,:)=0.
  tau (1:knchpf,:)=0.
  radov(1:knchpf,:)=0.

  CALL prfintl                                                   &
       (pav, psav, pssv, pcv, pemis, knpf, klenpf, knchpf, ksat, & 
        xxm, xxw, xxo, xxc, debyeprof, debyesv, psav_d,          &
        xxm_d, xxw_d, xxo_d, debyeprof_d, debyesv_d,             &
        nlevsf,fracps,xpath,surfw,surfw_d, &
        nlevcd,fracpc,ts,ts_d,temp,temp_d,wmix,wmix_d,ozon,ozon_d,cldw, &
        emis,ta,wmixs,surfp,cldp,cldf)


  CALL opdeptl                                                            &
       (knpf, kchan, kprof, knchpf, ksat, opdp, xxm, xxw, xxo, xxc,       &
       debyeprof, opdp_d, opdpref_d, zopdpref_d, xxc_d, debyeprof_d )

  CALL rttautl (kprof, knchpf, opdp, tau, tausfc, opdp_d, tau_d, tausfc_d,&
       nlevsf,fracps_d,fracps)

  CALL rtinttl &
   (kchan, kprof, knchpf, lcloud, prad, ptb, prad_d, ptb_d, ksat, pemis, &
   prefl, tau, tausfc, b, ba, bs, radov, rado, bdt, bdtr, fcld, radcl, tbcl, &
   pemis_d, prefl_d, tau_d, tausfc_d, b_d, ba_d, bs_d, radov_d, rado_d, &
   bdtr_d, fcld_d, radcl_d, tbcl_d,&
   nstype,nlevsf,fracps_d,&
   xpath,snad2,cnad2,czen,czen2,szen2,surfw,surfw_d,nlevcd,fracpc,fracpc_d,&
   temp,temp_d,ta,ta_d,ts,ts_d,cldf,emis,emis1_d)


  RETURN
END SUBROUTINE RTTOVTL
